@@ -1,6 +1,6 @@
 ;;; helm-bookmark.el --- Helm for Emacs regular Bookmarks. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -159,7 +159,7 @@
   "Toggle bookmark location visibility."
   (interactive)
   (with-helm-alive-p
-    (helm-attrset 'toggle-filename
+    (helm-set-attr 'toggle-filename
                   '(helm-bookmark-toggle-filename-1 . never-split))
     (helm-execute-persistent-action 'toggle-filename)))
 (put 'helm-bookmark-toggle-filename 'helm-only t)
@@ -464,9 +464,9 @@ If `browse-url-browser-function' is set to something else than
 
 (defclass helm-bookmark-override-inheritor (helm-source) ())
 
-(defmethod helm--setup-source ((source helm-bookmark-override-inheritor))
+(cl-defmethod helm--setup-source ((source helm-bookmark-override-inheritor))
   ;; Ensure `helm-source-in-buffer' method is called.
-  (call-next-method)
+  (cl-call-next-method)
   (setf (slot-value source 'action)
         (helm-append-at-nth
          (cl-loop for (name . action) in helm-type-bookmark-actions
@@ -737,10 +737,4 @@ if external addressbook-bookmark package is installed."
 
 (provide 'helm-bookmark)
 
-;; Local Variables:
-;; byte-compile-warnings: (not obsolete)
-;; coding: utf-8
-;; indent-tabs-mode: nil
-;; End:
-
 ;;; helm-bookmark.el ends here
