@@ -1,6 +1,6 @@
 ;;; helm-color.el --- colors and faces -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 (require 'helm-help)
 (require 'helm-elisp)
 
+(declare-function list-colors-display "facemenu")
+
 ;;; Customize Face
 ;;
 ;;
@@ -59,6 +61,7 @@
 ;;
 ;;
 (defun helm-colors-init ()
+  (require 'facemenu)
   (unless (helm-candidate-buffer)
     (save-selected-window
       (list-colors-display)
@@ -161,10 +164,4 @@
 
 (provide 'helm-color)
 
-;; Local Variables:
-;; byte-compile-warnings: (not obsolete)
-;; coding: utf-8
-;; indent-tabs-mode: nil
-;; End:
-
 ;;; helm-color.el ends here
