@@ -1,6 +1,6 @@
 ;;; helm-command.el --- Helm execute-exended-command. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -38,6 +38,10 @@
   :group 'helm-command
   :type 'boolean)
 
+(defcustom helm-M-x-fuzzy-match t
+  "Helm-M-x fuzzy matching when non nil."
+  :group 'helm-command
+  :type 'boolean)
 
 ;;; Faces
 ;;
@@ -111,25 +115,25 @@ algorithm."
           for cand in candidates
           for local-key  = (car (rassq cand local-map))
           for key        = (substitute-command-keys (format "\\[%s]" cand))
-          for sym        = (intern cand)
+          for sym        = (intern (if (consp cand) (car cand) cand))
           for disp       = (if (or (eq sym major-mode)
                                    (and (memq sym minor-mode-list)
                                         (boundp sym)
                                         (buffer-local-value sym helm-current-buffer)))
                                (propertize cand 'face 'helm-command-active-mode)
                              cand)
-          unless (get (intern (if (consp cand) (car cand) cand)) 'helm-only)
+          unless (or (get sym 'helm-only) (get sym 'no-helm-mx))
           collect
           (cons (cond ((and (string-match "^M-x" key) local-key)
-                       (format "%s (%s)"
+                       (format "%s %s"
                                disp (propertize
-                                     local-key
-                                     'face 'helm-M-x-key)))
+                                     " " 'display
+                                     (propertize local-key 'face 'helm-M-x-key))))
                       ((string-match "^M-x" key) disp)
-                      (t (format "%s (%s)"
+                      (t (format "%s %s"
                                  disp (propertize
-                                       key
-                                       'face 'helm-M-x-key))))
+                                       " " 'display
+                                       (propertize key 'face 'helm-M-x-key)))))
                 cand)
           into ls
           finally return
@@ -200,9 +204,8 @@ algorithm."
   (remove-hook 'helm-move-selection-after-hook
                'helm-M-x--move-selection-after-hook))
 
-(defclass helm-M-x-class (helm-source-sync helm-type-command)
-  ((match-dynamic :initform t)
-   (requires-pattern :initform 0)
+(defclass helm-M-x-class (helm-source-in-buffer helm-type-command)
+  ((requires-pattern :initform 0)
    (must-match :initform t)
    (filtered-candidate-transformer :initform 'helm-M-x-transformer-no-sort)
    (persistent-help :initform "Describe this command")
@@ -213,9 +216,9 @@ algorithm."
 (defun helm-M-x-read-extended-command (collection &optional predicate history)
   "Read or execute action on command name in COLLECTION or HISTORY.
 
-When `helm-M-x-use-completion-styles' is used, several actions as
-of `helm-type-command' are used and executed from here, otherwise
-this function returns the command as a symbol.
+When `helm-M-x-use-completion-styles' is used, Emacs
+`completion-styles' mechanism is used, otherwise standard helm
+completion and helm fuzzy matching are used together.
 
 Helm completion is not provided when executing or defining kbd
 macros.
@@ -225,23 +228,29 @@ suitable for `try-completion'.  Arg PREDICATE is a function that
 default to `commandp' see also `try-completion'.  Arg HISTORY
 default to `extended-command-history'."
   (let* ((helm--mode-line-display-prefarg t)
-         (minibuffer-completion-confirm t)
          (pred (or predicate #'commandp))
-         (metadata (unless (assq 'flex completion-styles-alist)
-                     '(metadata (display-sort-function
-                                 .
-                                 (lambda (candidates)
-                                   (sort candidates #'helm-generic-sort-fn))))))
+         (helm-fuzzy-sort-fn (lambda (candidates _source)
+                               ;; Sort on real candidate otherwise
+                               ;; "symbol (<binding>)" is used when sorting.
+                               (helm-fuzzy-matching-default-sort-fn-1 candidates t)))
          (sources `(,(helm-make-source "Emacs Commands history" 'helm-M-x-class
-                       :candidates (helm-dynamic-completion
-                                    ;; A list of strings.
-                                    (or history extended-command-history)
-                                    (lambda (str) (funcall pred (intern-soft str)))
-                                    nil 'nosort t))
+                       :data (lambda ()
+                               (helm-comp-read-get-candidates
+                                ;; History should be quoted to
+                                ;; force `helm-comp-read-get-candidates'
+                                ;; to use predicate against
+                                ;; symbol and not string.
+                                (or history 'extended-command-history)
+                                ;; Ensure using empty string to
+                                ;; not defeat helm matching fns [1]
+                                pred nil nil ""))
+                       :fuzzy-match helm-M-x-fuzzy-match)
                     ,(helm-make-source "Emacs Commands" 'helm-M-x-class
-                       :candidates (helm-dynamic-completion
-                                    collection pred
-                                    nil metadata t))))
+                       :data (lambda ()
+                               (helm-comp-read-get-candidates
+                                ;; [1] Same comment as above.
+                                collection pred nil nil ""))
+                       :fuzzy-match helm-M-x-fuzzy-match)))
          (prompt (concat (cond
                           ((eq helm-M-x-prefix-argument '-) "- ")
                           ((and (consp helm-M-x-prefix-argument)
@@ -253,7 +262,7 @@ default to `extended-command-history'."
                            (format "%d " helm-M-x-prefix-argument)))
                          "M-x ")))
     (setq helm-M-x--timer (run-at-time 1 0.1 'helm-M-x--notify-prefix-arg))
-    ;; Fix Issue #2250, add `helm-move-selection-after-hook' which
+    ;; Fix Bug#2250, add `helm-move-selection-after-hook' which
     ;; reset prefix arg to nil only for this helm session.
     (add-hook 'helm-move-selection-after-hook
               'helm-M-x--move-selection-after-hook)
@@ -301,7 +310,7 @@ Save COMMAND to `extended-command-history'."
   (when command
     ;; Avoid having `this-command' set to *exit-minibuffer.
     (setq this-command command
-          ;; Handle C-x z (repeat) Issue #322
+          ;; Handle C-x z (repeat) Bug#322
           real-this-command command)
     ;; If helm-M-x is called with regular emacs completion (kmacro)
     ;; use the value of arg otherwise use helm-current-prefix-arg.
@@ -351,10 +360,4 @@ You can get help on each command by persistent action."
 
 (provide 'helm-command)
 
-;; Local Variables:
-;; byte-compile-warnings: (not obsolete)
-;; coding: utf-8
-;; indent-tabs-mode: nil
-;; End:
-
 ;;; helm-command.el ends here
