@@ -1,6 +1,6 @@
 ;;; helm-elisp.el --- Elisp symbols completion for helm. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -397,7 +397,7 @@ the same time to variable and a function."
         for annot = (cl-typecase sym
                       (command " (Com)")
                       (class   " (Class)")
-                      (generic " (Gen)")
+                      (cl-generic " (Gen)")
                       (fbound  " (Fun)")
                       (bound   " (Var)")
                       (face    " (Face)"))
@@ -625,6 +625,7 @@ double quote."
     :fuzzy-match helm-apropos-fuzzy-match
     :filtered-candidate-transformer (and (null helm-apropos-fuzzy-match)
                                          'helm-apropos-default-sort-fn)
+    :display-to-real 'helm-symbolify
     :nomark t
     :persistent-action (lambda (candidate)
                          (helm-elisp--persistent-help
@@ -638,12 +639,13 @@ double quote."
             (helm-apropos-init (lambda (x)
                                  (and (fboundp x)
                                       (not (commandp x))
-                                      (not (generic-p x))
+                                      (not (cl-generic-p x))
                                       (not (class-p x))))
                                default))
     :fuzzy-match helm-apropos-fuzzy-match
     :filtered-candidate-transformer (and (null helm-apropos-fuzzy-match)
                                          'helm-apropos-default-sort-fn)
+    :display-to-real 'helm-symbolify
     :persistent-action (lambda (candidate)
                          (helm-elisp--persistent-help
                           candidate 'helm-describe-function))
@@ -673,7 +675,7 @@ double quote."
   (helm-build-in-buffer-source "Generic functions"
     :init (lambda ()
             (helm-apropos-init (lambda (x)
-                                 (generic-p x))
+                                 (cl-generic-p x))
                                default))
     :fuzzy-match helm-apropos-fuzzy-match
     :filtered-candidate-transformer (and (null helm-apropos-fuzzy-match)
@@ -693,7 +695,7 @@ double quote."
     (cond ((class-p sym)
            (setq fn #'helm-describe-function
                  src-name "Describe class"))
-          ((generic-p sym)
+          ((cl-generic-p sym)
            (setq fn #'helm-describe-function
                  src-name "Describe generic function"))
           ((fboundp sym)
@@ -968,10 +970,4 @@ a string, i.e. the `symbol-name' of any existing symbol."
 
 (provide 'helm-elisp)
 
-;; Local Variables:
-;; byte-compile-warnings: (not obsolete)
-;; coding: utf-8
-;; indent-tabs-mode: nil
-;; End:
-
 ;;; helm-elisp.el ends here
