@@ -1,6 +1,6 @@
 ;;; helm-locate.el --- helm interface for locate. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -356,7 +356,7 @@ Sort is done on basename of CANDIDATES."
    (group :initform 'helm-locate)))
 
 ;; Override helm-type-file class keymap.
-(defmethod helm--setup-source :after ((source helm-locate-override-inheritor))
+(cl-defmethod helm--setup-source :after ((source helm-locate-override-inheritor))
   (setf (slot-value source 'keymap) helm-locate-map))
 
 (defvar helm-source-locate
@@ -422,14 +422,14 @@ Sort is done on basename of CANDIDATES."
      (if (string-match-p "\\`fd" helm-locate-recursive-dirs-command)
          (format helm-locate-recursive-dirs-command
                  ;; fd pass path at end.
-                 (helm-attr 'subdir) (helm-attr 'basedir))
+                 (helm-get-attr 'subdir) (helm-get-attr 'basedir))
        (format helm-locate-recursive-dirs-command
 	       (if (string-match-p "\\`es" helm-locate-recursive-dirs-command)
                    ;; Fix W32 paths.
 		   (replace-regexp-in-string
-                    "/" "\\\\\\\\" (helm-attr 'basedir))
-                 (helm-attr 'basedir))
-	       (helm-attr 'subdir)))
+                    "/" "\\\\\\\\" (helm-get-attr 'basedir))
+                 (helm-get-attr 'basedir))
+	       (helm-get-attr 'subdir)))
      nil t nil)
     (buffer-string)))
 
@@ -473,10 +473,4 @@ Where db_path is a filename matched by
 
 (provide 'helm-locate)
 
-;; Local Variables:
-;; byte-compile-warnings: (not obsolete)
-;; coding: utf-8
-;; indent-tabs-mode: nil
-;; End:
-
 ;;; helm-locate.el ends here
