@@ -1,6 +1,6 @@
 ;;; helm-net.el --- helm browse url and search web. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -241,7 +241,7 @@ Can be \"-new-tab\" (default) or \"-new-window\"."
     :candidates (lambda ()
                   (funcall helm-google-suggest-default-function))
     :action 'helm-google-suggest-actions
-    :volatile t
+    :match-dynamic t
     :keymap helm-map
     :requires-pattern 3))
 
@@ -259,7 +259,7 @@ Can be \"-new-tab\" (default) or \"-new-window\"."
 
 (defvar helm-browse-url-chromium-program "chromium-browser")
 (defvar helm-browse-url-uzbl-program "uzbl-browser")
-(defvar helm-browse-url-next-program "next")
+(defvar helm-browse-url-nyxt-program "nyxt")
 (defvar helm-browse-url-conkeror-program "conkeror")
 (defvar helm-browse-url-opera-program "opera")
 (defvar helm-browse-url-default-browser-alist
@@ -345,10 +345,10 @@ NOTE: Probably not supported on some systems (e.g., Windows)."
   (helm-generic-browser url helm-browse-url-conkeror-program))
 
 ;;;###autoload
-(defun helm-browse-url-next (url &optional _ignore)
+(defun helm-browse-url-nyxt (url &optional _ignore)
   "Browse URL with next browser."
   (interactive "sURL: ")
-  (helm-generic-browser url helm-browse-url-next-program))
+  (helm-generic-browser url helm-browse-url-nyxt-program))
 
 (defun helm-browse-url-default-browser (url &rest args)
   "Find the first available browser and ask it to load URL."
@@ -430,10 +430,4 @@ NOTE: Probably not supported on some systems (e.g., Windows)."
 
 (provide 'helm-net)
 
-;; Local Variables:
-;; byte-compile-warnings: (not obsolete)
-;; coding: utf-8
-;; indent-tabs-mode: nil
-;; End:
-
 ;;; helm-net.el ends here
