@@ -1,6 +1,6 @@
 ;;; helm-types.el --- Helm types classes and methods. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015 ~ 2019  Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2015 ~ 2020  Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; Author: Thierry Volpiatto <thierry.volpiatto@gmail.com>
 ;; URL: http://github.com/emacs-helm/helm
@@ -23,6 +23,7 @@
 
 (require 'cl-lib)
 (require 'eieio)
+(eval-when-compile (require 'helm-source))
 
 (defvar helm-map)
 (defvar helm-mode-line-string)
@@ -36,7 +37,7 @@
 (defclass helm-type-file (helm-source) ()
   "A class to define helm type file.")
 
-(defmethod helm-source-get-action-from-type ((object helm-type-file))
+(cl-defmethod helm-source-get-action-from-type ((object helm-type-file))
   (slot-value object 'action))
 
 (defun helm-actions-from-type-file ()
@@ -75,7 +76,7 @@
 
 (defcustom helm-type-file-actions
   (helm-make-actions
-    "Find file"                               'helm-find-many-files
+    "Find file"                               'helm-find-file-or-marked
     "Find file as root"                       'helm-find-file-as-root
     "Find file other window"                  'helm-find-files-other-window
     "Find file other frame"                   'find-file-other-frame
@@ -105,9 +106,9 @@
   :group 'helm-files
   :type '(alist :key-type string :value-type function))
 
-(defmethod helm--setup-source :primary ((_source helm-type-file)))
+(cl-defmethod helm--setup-source ((_source helm-type-file)))
 
-(defmethod helm--setup-source :before ((source helm-type-file))
+(cl-defmethod helm--setup-source :before ((source helm-type-file))
   (setf (slot-value source 'action) 'helm-type-file-actions)
   (setf (slot-value source 'persistent-help) "Show this file")
   (setf (slot-value source 'action-transformer)
@@ -145,12 +146,12 @@
   :type '(alist :key-type string
                    :value-type function))
 
-(defmethod helm-source-get-action-from-type ((object helm-type-bookmark))
+(cl-defmethod helm-source-get-action-from-type ((object helm-type-bookmark))
   (slot-value object 'action))
 
-(defmethod helm--setup-source :primary ((_source helm-type-bookmark)))
+(cl-defmethod helm--setup-source ((_source helm-type-bookmark)))
 
-(defmethod helm--setup-source :before ((source helm-type-bookmark))
+(cl-defmethod helm--setup-source :before ((source helm-type-bookmark))
   (setf (slot-value source 'action) 'helm-type-bookmark-actions)
   (setf (slot-value source 'keymap) helm-bookmark-map)
   (setf (slot-value source 'mode-line) (list "Bookmark(s)" helm-mode-line-string))
@@ -173,7 +174,7 @@
    'switch-to-buffer-other-frame
    (lambda () (and (fboundp 'tab-bar-mode)
                    "Switch to buffer other tab `C-c C-t'"))
-   'switch-to-buffer-other-tab
+   'helm-buffers-switch-to-buffer-other-tab
    "Switch to buffer at line number"
    'helm-switch-to-buffer-at-linum
    "Browse project `C-x C-d'"
@@ -200,18 +201,18 @@
   :group 'helm-buffers
   :type '(alist :key-type string :value-type function))
 
-(defmethod helm-source-get-action-from-type ((object helm-type-buffer))
+(cl-defmethod helm-source-get-action-from-type ((object helm-type-buffer))
   (slot-value object 'action))
 
-(defmethod helm--setup-source :primary ((_source helm-type-buffer)))
+(cl-defmethod helm--setup-source ((_source helm-type-buffer)))
 
-(defmethod helm--setup-source :before ((source helm-type-buffer))
+(cl-defmethod helm--setup-source :before ((source helm-type-buffer))
   (setf (slot-value source 'action) 'helm-type-buffer-actions)
   (setf (slot-value source 'persistent-help) "Show this buffer")
   (setf (slot-value source 'mode-line)
         ;; Use default-value of `helm-mode-line-string' in case user
         ;; starts with a helm buffer as current-buffer otherwise the
-        ;; local value of this helm buffer is used (issues #1517,#2377).
+        ;; local value of this helm buffer is used (bug#1517, bug#2377).
         (list "Buffer(s)" (default-value 'helm-mode-line-string)))
   (setf (slot-value source 'filtered-candidate-transformer)
         '(helm-skip-boring-buffers
@@ -226,7 +227,9 @@
 (defcustom helm-type-function-actions
   (helm-make-actions
    "Describe function" 'helm-describe-function
-   "Find function" 'find-function
+   "Find function" (if (fboundp 'xref-find-definitions)
+                       ;; Find-function is broken in emacs-27.
+                       'xref-find-definitions 'find-function)
    "Info lookup" 'helm-info-lookup-symbol
    "Debug on entry" 'debug-on-entry
    "Cancel debug on entry" 'cancel-debug-on-entry
@@ -239,7 +242,7 @@
   ;; autoloaded (like untrace-function).
   :type '(alist :key-type string :value-type symbol))
 
-(defmethod helm-source-get-action-from-type ((object helm-type-function))
+(cl-defmethod helm-source-get-action-from-type ((object helm-type-function))
   (slot-value object 'action))
 
 (defun helm-actions-from-type-function ()
@@ -247,9 +250,9 @@
     (helm--setup-source source)
     (helm-source-get-action-from-type source)))
 
-(defmethod helm--setup-source :primary ((_source helm-type-function)))
+(cl-defmethod helm--setup-source ((_source helm-type-function)))
 
-(defmethod helm--setup-source :before ((source helm-type-function))
+(cl-defmethod helm--setup-source :before ((source helm-type-function))
   (setf (slot-value source 'action) 'helm-type-function-actions)
   (setf (slot-value source 'action-transformer)
         'helm-transform-function-call-interactively)
@@ -276,9 +279,9 @@
   :group 'helm-command
   :type '(alist :key-type string :value-type symbol))
 
-(defmethod helm--setup-source :primary ((_source helm-type-command)))
+(cl-defmethod helm--setup-source ((_source helm-type-command)))
 
-(defmethod helm--setup-source :before ((source helm-type-command))
+(cl-defmethod helm--setup-source :before ((source helm-type-command))
   (setf (slot-value source 'action) 'helm-type-command-actions)
   (setf (slot-value source 'coerce) 'helm-symbolify)
   (setf (slot-value source 'persistent-action) 'helm-M-x-persistent-action)
@@ -306,9 +309,9 @@
   :group 'helm-elisp
   :type '(alist :key-type string :value-type function))
 
-(defmethod helm--setup-source :primary ((_source helm-type-timers)))
+(cl-defmethod helm--setup-source ((_source helm-type-timers)))
 
-(defmethod helm--setup-source :before ((source helm-type-timers))
+(cl-defmethod helm--setup-source :before ((source helm-type-timers))
   (setf (slot-value source 'action) 'helm-type-timers-actions)
   (setf (slot-value source 'persistent-action)
         (lambda (tm)
@@ -328,10 +331,4 @@
 
 (provide 'helm-types)
 
-;; Local Variables:
-;; byte-compile-warnings: (not obsolete)
-;; coding: utf-8
-;; indent-tabs-mode: nil
-;; End:
-
 ;;; helm-types.el ends here
